#include<stdio.h>
#include<math.h>
#include<X11/Xlib.h>
#include<X11/Xutil.h>
#include"incim.h"

#include "vector.h"
#include "matrix.h"
#include "color.h"
#include "plight.h"
#include "ellipse.h"
#include "shading.h"

#define MAXELLIPSE 512
#define SCREEN_SIZE 256

main(){

	Color bgcol;		/* 背景色=球が見えなかったときの色 */
	Color tmpcol;		/* 輝度を求めるための作業変数 */
	Vector eyepos;		/* 視点の座標 */
	Vector endpos;		/* 視線の終点の座標 */
	Vector eyevec;		/* 視線方向ベクトル */
	int ix,iy;			/* スクリーン上の位置 */
	double t;			/* 視点から交点までの距離=２次方程式の解 */
	double mint;		/* 視点から交点までの最短距離 */
	int    mins;    	/* 視線と交差する球の番号 */

	int Screen;
	int MaxX,MaxY;

  	double ia;			/* 環境光の強度 */
  	double id;			/* 拡散反射光の強度 */
  	double is;			/* 鏡面反射光の強度 */

	Vector vi;			/* 交点の座標 */
	Vector nv;			/* 法線ベクトル(N) */

	int	i;		/* 処理中の楕円の番号 */
	int	n;		/* シーンを構成する楕円の数 */
	Ellipse ellipses[MAXELLIPSE];
	/* 点光源 */
	PointLight pointlight;


	/* 視点の設定 */
	eyepos.x=0;
	eyepos.y=0;
	eyepos.z=700;
	/*
 	* 楕円の方程式や反射モデルのデータなどをキーボード入力する
 	*/
	scanf( "%lf %lf %lf %lf\n",
		&pointlight.pos.x,&pointlight.pos.y,&pointlight.pos.z,
		&pointlight.ii);	/* 点光源の座標と強度 */

	/* 背景色(球が見えなかったときの色)を設定する */
	scanf( "%d %d %d\n",&bgcol.r,&bgcol.g,&bgcol.b );

	scanf( "%d",&n );	/* シーンを構成する楕円の数 */

	for(i=0;i<n;i++){
		scanf("%lf %lf %lf %lf %lf %lf %d %d %d %lf %lf %lf %lf\n",
		      &ellipses[i].pos.x,&ellipses[i].pos.y,&ellipses[i].pos.z,
			  &ellipses[i].a,&ellipses[i].b,&ellipses[i].c,
		      &ellipses[i].color.r,&ellipses[i].color.g,&ellipses[i].color.b,	
		      &ellipses[i].ka,&ellipses[i].kd,&ellipses[i].ks,&ellipses[i].n );
	}

/* ウインドウの設定 */

	/*scanf("%d\n",&Screen);*/
	Screen=SCREEN_SIZE;
	MaxX = (Screen-1)/2;
	MaxY = MaxX;
	openwindow(Screen,Screen,"ray22.c",1);

	/*
 	* 光線追跡
 	*/
	for(iy=MaxY;iy>=-MaxY;iy--){
		for (ix=-MaxX; ix<=MaxX; ix++){ 
			endpos.x=(double)ix;
			endpos.y=(double)iy;
			endpos.z=0;
			
			eyevec=sub(endpos,eyepos);
			eyevec=normalize(eyevec);
			/*
			* 画素毎の初期設定
			*/
			mint=1000000;	/* 視点からの距離を無限遠方に設定する */
			mins=-1;	/* 視線と交差する楕円の番号を-1に設定する */
			/* 楕円の番号は0からであるので視線と交差する楕円はないことになる */
			tmpcol=bgcol;/* 視線と交差しない場合の背景の色を設定する */

			/*
			* すべての楕円に対して，視線と各球とが交差するかどうか，調べる
			* 交差すれば交点を求める
			*/
			for (i=0; i<n; i++){ 
				t=calcEllipseT(ellipses[i],eyepos,eyevec);
				/* 求めた距離がそれまでの最短距離であった場合 */
				if ((t>0) && (t<=mint)){ 
					mint=t;	/* 最短距離を更新する */
					mins=i;	/* 楕円の番号を更新する */
				}
			}
			/*
			* 交点があった場合
			*/
			if (mins>=0){
				/* 交点の座標を求める */
				vi=add(eyepos,scale(mint,eyevec));
				/* 交点での法線ベクトルを求める */
				nv=calcEllipseNorm(ellipses[mins],vi);
				
				calcShading(ellipses[mins],pointlight,vi,nv,eyepos,&ia,&id,&is);
			}else{ 
				id=is=0;	/* 拡散反射光=鏡面反射光の強度=0 */
			}
      		/* 輝度計算 */
			if (mins>=0){
				tmpcol.r=(int)(ia*255.0+id*ellipses[mins].color.r+is*255.0+0.5); 
				tmpcol.g=(int)(ia*255.0+id*ellipses[mins].color.g+is*255.0+0.5);
				tmpcol.b=(int)(ia*255.0+id*ellipses[mins].color.b+is*255.0+0.5);
				if(tmpcol.r>255) tmpcol.r=255; if(tmpcol.r<0) tmpcol.r=0;
				if(tmpcol.g>255) tmpcol.g=255; if(tmpcol.g<0) tmpcol.g=0;
				if(tmpcol.b>255) tmpcol.b=255; if(tmpcol.b<0) tmpcol.b=0;
			}
			/* 画素の色を設定する  */
			setfgcolorbyRGB(tmpcol.r,tmpcol.g,tmpcol.b);
			/* 画素を描画する */
			setpixel(ix+MaxX,-iy+MaxY+1);
		}
	}	/* End of Loop */
  	/*
   	* Window保持
   	*/
	keepwindow();

	closewindow();
}	/* End of "main" */
